-- atlas for sprites in a grid (tilesets)

grid_atlas = class {
	init = function(self, image, w, h, border)

		self.width = w
		self.height = h
		self.border = border
		
		self.image = gfx[image]
		self.quad = slice(image, w, h, border)

	end;

	draw = function(self, entry, x, y, r, sx, sy, ox, oy, kx, ky)

		if not self.quad[entry] then return end
		love.graphics.draw(self.image, self.quad[entry], x, y, r, sx, sy, ox, oy, kx, ky)

	end;

}

-- atlas for packed spritesheets generated by shoebox

packed_atlas = class {

	init = function(self, image, quad)

		self.image = image

		self.quad = {}

		for i in ipairs(quad) do
			local q = quad[i]
			-- q: x, y, w, h
			self.quad[i] = love.graphics.newQuad(q[1], q[2], q[3], q[4], self.image:getDimensions())
		end

	end;

	draw = function(self, entry, x, y, r, sx, sy, ox, oy, kx, ky)

		if not self.quad[entry] then return end
		love.graphics.draw(self.image, self.quad[entry], x, y, r, sx, sy, ox, oy, kx, ky)

	end;

	getWidth = function(self, entry)
		if not self.quad[entry] then return end
		local x, y, w, h = self.quad[entry]:getViewport()
		return w
	end;

	getHeight = function(self, entry)
		if not self.quad[entry] then return end
		local x, y, w, h = self.quad[entry]:getViewport()
		return h
	end;

	getDimensions = function(self, entry)
		if not self.quad[entry] then return end
		local x, y, w, h = self.quad[entry]:getViewport()
		return w, h
	end;

}

