function scrub_filename(str)

	return str:gsub("[/\\?*:|\"><.]")

end;

-- slice an image into quads

function slice(image, w, h, border)
	
		if not w or not h then return end

		local image = gfx[image]
		border = border or 0

		local quads = {}

		local i = 0
		for y = 0, image:getHeight()-1, h+border*2 do
			for x = 0, image:getWidth()-1, w+border*2 do
				i = i + 1
				quads[i] = love.graphics.newQuad(x+border, y+border, w, h, image:getWidth(), image:getHeight())
			end
		end	

		return quads
		
end;

-- check if a position is within a box

function within(x, y, left, top, right, bottom)
	if x > left and x < right and y > top and y < bottom then return true end
end

-- !!! resource loading functions

function load_lua(directory) -- require .lua from dir (not subdirs)

	local dir = "/" .. directory .. "/"
	local files = love.filesystem.getDirectoryItems(dir)
	for i, file in ipairs(files) do
		if love.filesystem.isFile(dir .. file) then
			print(dir .. file)
			require(directory .. "." .. getBasename(file))
		end
	end

end

function load_images(directory) -- load images from dir (not subdirs)

	local tbl = {}
	local tbl2 = {}

	local dir = "/" .. directory .. "/"
	local files = love.filesystem.getDirectoryItems(dir)
	for i, file in ipairs(files) do
		if love.filesystem.isFile(dir .. file) then
			local base, ext = getBasenameAndExtension(file)
			if ext == "png" then
				tbl[base] = love.graphics.newImage(dir .. file)
				print("image: " .. base)
				if love.filesystem.isFile(dir .. base .. ".lua") then
					local quads = require(directory .. "." .. base)
					tbl2[base] = packed_atlas(tbl[base], quads)
				end
			end
		end

	end
	
	return tbl, tbl2

end

function load_atlases(directory)
end

function load_audio(directory, type) -- audio blah blah blah

	local tbl = {}

	local dir = "/" .. directory .. "/"
	local files = love.filesystem.getDirectoryItems(dir)
	for i, file in ipairs(files) do
		if love.filesystem.isFile(dir .. file) then
			local base, ext = getBasenameAndExtension(file)
			if ext == "ogg" or ext == "wav" then
				tbl[base] = love.audio.newSource(dir .. file, type)
				print("audio: " .. base)
			end
		end
	end
	
	return tbl

end

-- !!! by kikito, love forums https://love2d.org/forums/viewtopic.php?f=4&t=11145#p67016

function getBasename(filename)
  return filename:match("^([^%.]*)%.?") -- "myfile.lua" -> "myfile"
end

function getBasenameAndExtension(filename)
  return filename:match("^([^%.]*)%.?(.*)$") -- "myfile.lua" -> "myfile", "lua"
end

-- !!!

function math.round(num, idp) -- from lua-users wiki
  return tonumber(string.format("%." .. (idp or 0) .. "f", num))
end

function math.clamp(_min, _num, _max)
	return math.min(math.max(_num, _min), _max)
end