function love.load()

	-- basic fonts

	font = {
		tiny = love.graphics.newFont(11),
		lesstiny = love.graphics.newFont(12),
		medium = love.graphics.newFont(16),
		big = love.graphics.newFont(24),
		steps_12 = love.graphics.newFont("font/Steps-Mono.otf", 12),
		steps_18 = love.graphics.newFont("font/Steps-Mono.otf", 18),
		steps_36 = love.graphics.newFont("font/Steps-Mono.otf", 36)

	} 

	-- libs
	
	class = require "lib.hump.class"
	timer = require "lib.hump.timer"
	colour = require "lib.colors"

	-- audio playing
	require "audio"
	sfxlist = {}
	streamlist = {}
	str_channel = {}

	-- various other functions
	require "func"

	-- load resources
	print("=== load resources ===")
	gfx = {}
	atlas = {}
	load_lua("class") -- reqs all the classes
	gfx, atlas = load_images("gfx")
	stream = load_audio("stream", "stream")
	sfx = load_audio("sfx", "static")

	controller = { 
		keyboard = {
			left = {"left"},
			right = {"right"},
			up = {"up"},
			down = {"down"},
			fire = {"space"},
			mode = {"tab"},
			muffle = {"backspace"},
			power = {"p"},
			warp = {"return"},
			help = {"f1", "t"},
			quit = {"escape"},
			hue = {"h"},
			saturation = {"s"},
			turbo = {"lshift", "rshift"},
			lightness = {"l"},
			col_black = {"1"},
			col_white = {"2"},
			col_red = {"3"},
			col_yellow = {"4"},
			col_green = {"5"},
			col_cyan = {"6"},
			col_blue = {"7"},
			col_magenta = {"8"},
			col_grey = {"9"},
			col_wild = {"0"},
			screenshot = {"f5", "y"},
			clear = {"f2", "x"}
		},
		states = {}, 
		tap = {}
	}

	canvas = love.graphics.newCanvas()
	canvas_final = love.graphics.newCanvas()
	setup_cf = true

	gfx.noise:setWrap("repeat", "repeat")
	noise_quad = love.graphics.newQuad(0, 0, love.graphics.getWidth(), love.graphics.getHeight(), 512, 512)
	things = {player(love.graphics.getWidth()/2, love.graphics.getHeight()/2)}

	show_splash = true
	splash_sine = 0
	show_message = 0
	current_message = "everybody loves to blomby"

	audio.stream(1, "triloop", true, 1, 0)
	audio.stream(2, "triloop", true, 1, 0)
	audio.stream(3, "triloop", true, 1, 0)

end

function draw_splash()
	if not show_splash then return end
	local txt1 = { 	"to speed up and slow down\t",
					"\nto rotate left and right\t",
					"\nto engage afterburners\t",
					"\n",
					"\nto fire the blomby cannon\t",
					"\nto switch blomby cannon modes\t",
					"\n",
					"\nto toggle the megamuffler\t",
					"\nto engage hyperwarp\t",
					"\nto enable power-saving mode\t",
					"\n",
					"\nto select between energy crystals\t",
					"\nto alter energy field parameters\t",
					"\n",
					"\nto show and hide this text\t",
					"\nto flood sector with blomby energy\t",
					"\nto store readings in datacube\t",
					"\nto leave the blombyverse\t"}
	local txt2 = { "\tup/down",
					"\n\tleft/right",
					"\n\tshift",
					"\n",
					"\n\tspace",
					"\n\ttab",
					"\n",
					"\n\tbackspace",
					"\n\tenter",
					"\n\tP",
					"\n",
					"\n\t0-9",
					"\n\tH/S/L",
					"\n",
					"\n\tF1 -or- T",
					"\n\tF2 -or- X",
					"\n\tF5 -or- Y",
					"\n\tescape"}

	local sx = 1
	if love.graphics.getHeight() < 700 then
		sx = love.graphics.getHeight() / 700
	end
	love.graphics.push()
	--love.graphics.line(love.graphics.getWidth()/2, 0, love.graphics.getWidth()/2, love.graphics.getHeight())
	love.graphics.scale(1, sx)
	love.graphics.setFont(font.steps_36)
	love.graphics.setColor(0, 0, 0)
	local sine = math.sin(splash_sine)*10
	love.graphics.printf("blomby cannon", 2, love.graphics.getHeight()*0.10+2+sine, love.graphics.getWidth(), "center")
	love.graphics.setColor(255, 255, 255)
	love.graphics.printf("blomby cannon", 0, love.graphics.getHeight()*0.10+sine, love.graphics.getWidth(), "center")
	love.graphics.setFont(font.steps_18)
	love.graphics.setColor(0, 0, 0)
	love.graphics.printf(txt1, 2, love.graphics.getHeight()*0.25+2, love.graphics.getWidth()*0.50, "right")
	love.graphics.printf(txt2, love.graphics.getWidth()*0.5+2, love.graphics.getHeight()*0.25+2, love.graphics.getWidth()*0.5, "left")
	love.graphics.setColor(255, 255, 255)
	love.graphics.printf(txt1, 0, love.graphics.getHeight()*0.25, love.graphics.getWidth()*0.50, "right")
	love.graphics.printf(txt2, love.graphics.getWidth()*0.5, love.graphics.getHeight()*0.25, love.graphics.getWidth()*0.5, "left")
	love.graphics.pop()

end

function draw_message()
	if show_message <= 0 then return end
	love.graphics.setFont(font.steps_18)
	love.graphics.setColor(0, 0, 0, 230)
	love.graphics.rectangle("fill", 0, love.graphics.getHeight()-font.steps_18:getHeight()*2, love.graphics.getWidth(), 900)
	love.graphics.setColor(255, 255, 255)
	love.graphics.printf(current_message, 0, love.graphics.getHeight()-font.steps_18:getHeight()*1.5, love.graphics.getWidth(), "center")
end

function set_message(msg)
	current_message = msg
	show_message = 3
end

function love.resize(w,h)
	canvas = love.graphics.newCanvas(love.graphics.getWidth(), love.graphics.getHeight(), "hdr")
	canvas_final = love.graphics.newCanvas()
	setup_cf = true
end

function love.update(dt)

	-- dt = math.min(dt, 0.5)
	-- uhh

	if show_message > 0 then show_message = show_message - dt end

	splash_sine = splash_sine + 5 * dt
	if splash_sine > 100 then splash_sine = splash_sine - 100 end

	-- hump timer

	timer.update(dt)

	-- audio stuff
	-- audio master volume
	love.audio.setVolume(audio.volume)

	-- change volume/pitch of stream playing on channel n if necessary
	-- remove finished streams

	for i in ipairs(streamlist) do
		streamlist[i]:setVolume(str_channel[i].volume)
		streamlist[i]:setPitch(str_channel[i].pitch)
		if streamlist[i]:isStopped() then table.remove(streamlist, i) end
	end

	-- remove finished sounds

	for i in ipairs(sfxlist) do
		if sfxlist[i]:isStopped() then table.remove(sfxlist, i) end
	end	

	-- controller
	
	for k,v in pairs(controller.keyboard) do	
		local anyhit = false
		for i in pairs(controller.keyboard[k]) do
			if love.keyboard.isDown(controller.keyboard[k][i]) then
				anyhit = true
			end
		end
		controller.states[k] = anyhit
	end

	-- things

	for i in ipairs(things) do
		if things[i].marked_for_death then table.remove(things, i) 
		elseif not things[i].disabled then
			things[i]:update(dt)
		end
	end

	--

	if controller.tap.screenshot then
		local screen = canvas_final:newImageData()
		local filename = os.date("%Y-%m-%d %H-%M-%S") .. ".png"
    	screen:encode("png", filename)
    	audio.multisquawk({"select1", "select2", "select3"})
    	set_message("saved " .. love.filesystem.getSaveDirectory() .. "/" .. filename)
	end

	if controller.tap.help then show_splash = not show_splash
	elseif	controller.tap.quit then love.event.quit() end

	controller.tap = {}

end

function love.draw()

	if setup_cf then
		love.graphics.setCanvas(canvas_final)
		love.graphics.clear(0, 0, 0, 255)
		love.graphics.setCanvas()
		setup_cf = false
	end

	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.draw(canvas_final)

	for i in ipairs(things) do
		if not things[i].disabled then
			things[i]:draw()
		end
	end

	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.setCanvas(canvas_final)
	love.graphics.setBlendMode("alpha", "premultiplied")
	love.graphics.draw(canvas)
	love.graphics.setBlendMode("alpha")
	love.graphics.setCanvas(canvas)
	love.graphics.clear(0, 0, 0, 0)
	love.graphics.setCanvas()

	draw_splash()
	draw_message()
	--love.graphics.print(love.timer.getFPS(), 0, 0)

end

function love.mousemoved(x, y, dx, dy)
end

function love.keypressed(key)
end

function love.keyreleased(key)

	for k,v in pairs(controller.keyboard) do	
		local anyhit = false
		for i in ipairs(controller.keyboard[k]) do
			if key == controller.keyboard[k][i] then
				anyhit = true
			end
		end
		controller.tap[k] = anyhit
	end

end

function love.textinput(t)
end

function love.mousereleased(x, y, button)
end

function love.mousepressed(x, y, button)
end

